from datetime import datetime

from dash import Dash
import pytest
import pandas as pd

from penn_chime.model.parameters import (
    Parameters,
    Disposition,
    Regions,
)
from penn_chime.model.sir import (
    Sir,
    build_floor_df,
)


class MockStreamlit:
    """Mock implementation of streamlit

    We just want to verify that st _attempted_ to render the right stuff
    so we store the input, and make sure that it matches what we expect
    """

    def __init__(self):
        self.render_store = []
        self.markdown = self.just_store_instead_of_rendering
        self.latex = self.just_store_instead_of_rendering
        self.subheader = self.just_store_instead_of_rendering

    def just_store_instead_of_rendering(self, inp, *args, **kwargs):
        self.render_store.append(inp)
        return None


@pytest.fixture()
def dash_app():
    return Dash()

@pytest.fixture
def mock_st():
    return MockStreamlit()


@pytest.fixture
def defaults():
    return Parameters(
        region=Regions(
            delaware=564696,
            chester=519293,
            montgomery=826075,
            bucks=628341,
            philly=1581000,
        ),
        current_date=datetime(year=2020, month=3, day=28),
        current_hospitalized=14,
        date_first_hospitalized=datetime(year=2020, month=3, day=7),
        doubling_time=4.0,
        hospitalized=Disposition.create(rate=0.025, days=7),
        icu=Disposition.create(rate=0.0075, days=9),
        infectious_days=14,
        n_days=60,
        market_share=0.15,
        mitigation_date=datetime(year=2020, month=3, day=28),
        recovered=0,
        relative_contact_rate=0.3,
        ventilated=Disposition.create(rate=0.005, days=10),
    )


@pytest.fixture
def param():
    return Parameters(
        current_date=datetime(year=2020, month=3, day=28),
        current_hospitalized=100,
        doubling_time=6.0,
        hospitalized=Disposition.create(rate=0.05, days=7),
        infectious_days=14,
        icu=Disposition.create(rate=0.02, days=9),
        market_share=0.05,
        mitigation_date=datetime(year=2020, month=3, day=28),
        n_days=60,
        population=500000,
        recovered=0,
        relative_contact_rate=0.15,
        ventilated=Disposition.create(rate=0.01, days=10),
    )


@pytest.fixture
def halving_param():
    return Parameters(
        current_date=datetime(year=2020, month=3, day=28),
        current_hospitalized=100,
        doubling_time=6.0,
        hospitalized=Disposition.create(rate=0.05, days=7),
        icu=Disposition.create(rate=0.02, days=9),
        infectious_days=14,
        market_share=0.05,
        mitigation_date=datetime(year=2020, month=3, day=28),
        n_days=60,
        population=500000,
        recovered=0,
        relative_contact_rate=0.7,
        ventilated=Disposition.create(rate=0.01, days=10),
    )


@pytest.fixture
def model(param):
    return Sir(param)


@pytest.fixture
def halving_model(halving_param):
    return Sir(halving_param)


@pytest.fixture
def admits_df():
    return pd.read_csv(
        "tests/by_doubling_time/2020-03-28_projected_admits.csv", parse_dates=["date"]
    )


@pytest.fixture
def admits_floor_df(param, admits_df):
    return build_floor_df(admits_df, param.dispositions.keys(), "admits_")


@pytest.fixture
def census_df():
    return pd.read_csv(
        "tests/by_doubling_time/2020-03-28_projected_census.csv", parse_dates=["date"]
    )

@pytest.fixture
def census_floor_df(param, census_df):
    return build_floor_df(census_df, param.dispositions.keys(), "census_")




"""Utils."""

from base64 import b64encode

import pandas as pd


def dataframe_to_base64(df: pd.DataFrame) -> str:
    """Converts a dataframe to a base64-encoded CSV representation of that data.

    This is useful for building datauris for use to download the data in the browser.

    Arguments:
        df: The dataframe to convert
    """
    csv = df.to_csv(index=False)
    b64 = b64encode(csv.encode()).decode()
    return b64



"""Command line interface."""

import os
import sys

from .model.parameters import Parameters
from .model.sir import Sir


def main():
    """Main."""
    p = Parameters.create(os.environ, sys.argv[1:])
    m = Sir(p)

    for df, name in (
        (m.sim_sir_w_date_df, "sim_sir_w_date"),
        (m.admits_df, "projected_admits"),
        (m.census_df, "projected_census"),
    ):
        df.to_csv(f"{p.current_date}_{name}.csv")



"""Constants."""

from datetime import date

"""
This reflects a date from which previously-run reports will no
longer match current results, indicating when users should
re-run their reports
"""
CHANGE_DATE = date(year=2020, month=4, day=8)
VERSION = 'v1.1.3'

DATE_FORMAT = "%b, %d"  # see https://strftime.org
DOCS_URL = "https://code-for-philly.gitbook.io/chime"

EPSILON = 1.0e-7

FLOAT_INPUT_MIN = 0.0001
FLOAT_INPUT_STEP = 0.1



"""setup for PyPI deploy
"""

from setuptools import find_packages, setup

VERSION = "1.1.3"  # update VERSION in constants.py
AUTHOR = "Predictive Healthcare @ Penn Medicine"
MAINTAINER = "Brian Ross, Code for Philly"
MAINTAINER_EMAIL = "ml@brianthomasross.com"
URL = "https://codeforphilly.github.io/chime/"
PROJECT_URLS={
    "Bug Reports": "https://github.com/CodeForPhilly/chime/issues",
    "Source": "https://github.com/CodeForPhilly/chime",
    "Documentation": "https://codeforphilly.github.io/chime/",
}
REQUIREMENTS = [
            "altair",
            "gspread",
            "numpy",
            "pandas",
            "pytest",
            "streamlit",
            "gspread",
            "oauth2client"
]
with open("README.rst") as rm:
    README = rm.read()
CLASSIFIERS = [
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ]


setup(
    name="penn-chime",
    version=VERSION,
    author=AUTHOR,
    author_email="",
    description="Covid-19 Hospital Impact Model for Epidemics",
    long_description=README,
    url=URL,
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    test_suite='tests',

    python_requires='>=3.7',
    entry_points={
        'console_scripts': ['penn_chime=penn_chime.cli:main'],
    },
    include_package_data=True,
)












"""Setup file for chime
"""
__version__ = "1.1.3"  # update VERSION in constants.py
__author__ = "Predictive Healthcare @ Penn Medicine"

from setuptools import setup, find_namespace_packages



